function wiener(filename, eta, sigma, lambda)

% definition of parameters
%filename = 'lena.jpg';	% input file
%eta	= 10.0;		% amplitude of white noise
%sigma	= 4.0;		% width of the Gaussian (size in pixels)
%lambda	= 0.0*eta*eta;	% smoothness parameter
%size	= 512;		% size of image (should be even)

info = imfinfo(filename);
size = floor(min(info.Width, info.Height)/2)*2;  % square image of even size
hsize	= size/2;	% half size of image 

% show original image
[rimage,M] = imread(filename);
image = rimage(:,:,1);
imagefig=figure;
set(imagefig,'Name','Image without distortion');
imshow(im2uint8(mat2gray(double(real(image)))));

% generate Gaussian and blur image
gauss = zeros(size,size);
if sigma>0.1
	gauss(1:size-1,1:size-1) = fspecial('gaussian',size-1,sigma);
else
	gauss(size/2,size/2) = 1;
end
fgauss = fft2(ifftshift(gauss));
fimage  = fft2(image,size,size);
dfimage = fimage.*fgauss;
dimage  = ifft2(dfimage,size,size);

% generate additive white noise
noise  = eta*randn(size);
fnoise = fft2(noise);
dimage  = dimage+noise;	% addition of white noise 
dimage(dimage>255) = 255;
dimage(dimage<0) = 0;

% cyclic shifting of image due to even image size
dimage_n = zeros(size,size);
dimage_n(2:size,2:size) = dimage(1:size-1,1:size-1);
dimage_n(1,:)         = dimage(size,:);
dimage_n(:,1)         = dimage(:,size);

% show noisy blurred image 
PSNR_image_gestoert=psnr(image,dimage_n)
blurfig=figure;
set(blurfig,'Name','Image distorted by Gaussian blur and noise');
imshow(im2uint8(mat2gray(double(real(dimage_n)))));

% compute Wiener filter: autocorrelation
	% autocorrelation of original image
Phi_bb	= fimage.*conj(fimage);	
	% autocorrelation of noise 
Phi_nn	= fnoise.*conj(fnoise);		% (true noise autocorrelation)
%Phi_scale = size*size*eta*eta; 
%Phi_nn	= Phi_scale*ones(size,size);	% (expected noise autocorr.)
	% smoothness term (u^2+v^2)
fLaplace = zeros(size,size);
if lambda > 0
	for i=1:1:hsize
	for j=1:1:hsize
% $(\nabla f)^2$ as penalty term
%		fLaplace(i,j) = (i-1)*(i-1) + (j-1)*(j-1);
%		fLaplace(i,513-j) = (i-1)*(i-1) + j*j;
%		fLaplace(513-i,j) = i*i + (j-1)*(j-1);
%		fLaplace(513-i,513-j) = i*i + j*j;
% $(\Delta f)^2$ as penalty term (yields biharmonic operator)
		fLaplace(i,j) = (i-1)^4 + 2*((i-1)*(j-1))^2 + (j-1)^4;
		fLaplace(i,size+1-j) = (i-1)^4 + 2*((i-1)*j)^2 + j^4;
		fLaplace(size+1-i,j) = i^4 + 2*(i*(j-1))^2 + (j-1)^4;
		fLaplace(size+1-i,size+1-j) = i^4 + 2*(i*j)^2 + j^4;
	end
	end
end

% inverse filter
denom	= ((fgauss.*conj(fgauss)) + (Phi_nn + lambda*fLaplace)./ Phi_bb);
fh_inv	= conj(fgauss) ./ denom;

% inverse transformation
dfimage = fft2(dimage);			% fft of distorted image
rest = real(ifft2(dfimage .* fh_inv));	% restoration and inverse fft	

% image clipping and visualization
rest(rest>255) = 255;
rest(rest<0) = 0;
restfig = figure;
set(restfig,'Name','Wiener filtered image');
imshow(im2uint8(mat2gray(double(real(rest)))));
PSNR_image_restauriert=psnr(image,rest)

% show wiener filter
fhabs = ifftshift(abs(fh_inv));
filterfig = figure;
set (filterfig,'Name','Reconstruction filter');
imshow(im2uint8(mat2gray(double(log(1+abs(fhabs))))));
filterfig = figure;
plot(1:1:512,fhabs(:,256));
